@@ -22,6 +22,8 @@ from pyvisa.compat import integer_types, OrderedDict
 from . import sessions
 from .common import logger
 
+StatusCode = constants.StatusCode
+
 
 class PyVisaLibrary(highlevel.VisaLibraryBase):
     """A pure Python backend for PyVISA.
@@ -42,25 +44,25 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
     try:
         from .serial import SerialSession
         logger.debug('SerialSession was correctly imported.')
-    except ImportError as e:
+    except Exception as e:
         logger.debug('SerialSession was not imported %s.' % e)
 
     try:
         from .usb import USBSession, USBRawSession
         logger.debug('USBSession and USBRawSession were correctly imported.')
-    except ImportError as e:
+    except Exception as e:
         logger.debug('USBSession and USBRawSession were not imported %s.' % e)
 
     try:
-        from .tcpip import TCPIPSession
+        from .tcpip import TCPIPInstrSession, TCPIPSocketSession
         logger.debug('TCPIPSession was correctly imported.')
-    except ImportError as e:
+    except Exception as e:
         logger.debug('TCPIPSession was not imported %s.' % e)
 
     try:
         from .gpib import GPIBSession
         logger.debug('GPIBSession was correctly imported.')
-    except ImportError as e:
+    except Exception as e:
         logger.debug('GPIBSession was not imported %s.' % e)
 
     @classmethod
@@ -122,7 +124,7 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
                      extra=self._logging_extra)
 
         try:
-            ret_value = constants.StatusCode(ret_value)
+            ret_value = StatusCode(ret_value)
         except ValueError:
             pass
 
@@ -162,7 +164,8 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
 
     # noinspection PyShadowingBuiltins
     def open(self, session, resource_name,
-             access_mode=constants.AccessModes.no_lock, open_timeout=constants.VI_TMO_IMMEDIATE):
+             access_mode=constants.AccessModes.no_lock,
+             open_timeout=constants.VI_TMO_IMMEDIATE):
         """Opens a session to the specified resource.
 
         Corresponds to viOpen function of the VISA library.
@@ -184,13 +187,105 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
         try:
             parsed = rname.parse_resource_name(resource_name)
         except rname.InvalidResourceName:
-            return 0, constants.StatusCode.error_invalid_resource_name
+            return 0, StatusCode.error_invalid_resource_name
 
         cls = sessions.Session.get_session_class(parsed.interface_type_const, parsed.resource_class)
 
-        sess = cls(session, resource_name, parsed)
+        sess = cls(session, resource_name, parsed, open_timeout)
 
-        return self._register(sess), constants.StatusCode.success
+        return self._register(sess), StatusCode.success
+
+    def assert_trigger(self, session, protocol):
+        """Asserts software or hardware trigger.
+
+        Corresponds to viAssertTrigger function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :param protocol: Trigger protocol to use during assertion. (Constants.PROT*)
+        :return: return value of the library call.
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return 0, constants.StatusCode.error_invalid_object
+        return sess.assert_trigger(protocol)
+
+    def clear(self, session):
+        """Clears a device.
+
+        Corresponds to viClear function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :return: return value of the library call.
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return 0, constants.StatusCode.error_invalid_object
+        return sess.clear()
+
+    def gpib_command(self, session, command_byte):
+        """Write GPIB command byte on the bus.
+
+        Corresponds to viGpibCommand function of the VISA library.
+        See: https://linux-gpib.sourceforge.io/doc_html/gpib-protocol.html#REFERENCE-COMMAND-BYTES
+
+        :param command_byte: command byte to send
+        :type command_byte: int, must be [0 255]
+        :return: return value of the library call
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            return self.sessions[session].gpib_command(command_byte)
+
+        except KeyError:
+            return constants.StatusCode.error_invalid_object
+
+    def assert_trigger(self, session, protocol):
+        """Asserts software or hardware trigger.
+
+        Corresponds to viAssertTrigger function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :param protocol: Trigger protocol to use during assertion. (Constants.PROT*)
+        :return: return value of the library call.
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            return self.sessions[session].trigger(protocol)
+
+        except KeyError:
+            return constants.StatusCode.error_invalid_object
+
+    def gpib_send_ifc(self, session):
+        """Pulse the interface clear line (IFC) for at least 100 microseconds.
+
+        Corresponds to viGpibSendIFC function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :return: return value of the library call.
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return 0, constants.StatusCode.error_invalid_object
+        return sess.gpib_send_ifc()
+
+    def read_stb(self, session):
+        """Reads a status byte of the service request.
+        Corresponds to viReadSTB function of the VISA library.
+        :param session: Unique logical identifier to a session.
+        :return: Service request status byte, return value of the library call.
+        :rtype: int, :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return 0, constants.StatusCode.error_invalid_object
+        return sess.read_stb()
 
     def close(self, session):
         """Closes the specified session, event, or find list.
@@ -206,7 +301,7 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
             if sess is not self:
                 sess.close()
         except KeyError:
-            return constants.StatusCode.error_invalid_object
+            return StatusCode.error_invalid_object
 
     def open_default_resource_manager(self):
         """This function returns a session to the Default Resource Manager resource.
@@ -216,7 +311,7 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
         :return: Unique logical identifier to a Default Resource Manager session, return value of the library call.
         :rtype: session, VISAStatus
         """
-        return self._register(self), constants.StatusCode.success
+        return self._register(self), StatusCode.success
 
     def list_resources(self, session, query='?*::INSTR'):
         """Returns a tuple of all connected devices matching query.
@@ -232,10 +327,7 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
 
         resources = rname.filter(resources, query)
 
-        if resources:
-            return resources
-
-        raise errors.VisaIOError(errors.StatusCode.error_resource_not_found.value)
+        return resources
 
     def read(self, session, count):
         """Reads data from device or interface synchronously.
@@ -250,9 +342,14 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
 
         # from the session handle, dispatch to the read method of the session object.
         try:
-            return self.sessions[session].read(count)
+            ret = self.sessions[session].read(count)
         except KeyError:
-            return constants.StatusCode.error_invalid_object
+            return 0, StatusCode.error_invalid_object
+
+        if ret[1] < 0:
+            raise errors.VisaIOError(ret[1])
+
+        return ret
 
     def write(self, session, data):
         """Writes data to device or interface synchronously.
@@ -268,9 +365,14 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
 
         # from the session handle, dispatch to the write method of the session object.
         try:
-            return self.sessions[session].write(data)
+            ret = self.sessions[session].write(data)
         except KeyError:
-            return constants.StatusCode.error_invalid_object
+            return 0, StatusCode.error_invalid_object
+
+        if ret[1] < 0:
+            raise errors.VisaIOError(ret[1])
+
+        return ret
 
     def get_attribute(self, session, attribute):
         """Retrieves the state of an attribute.
@@ -285,7 +387,7 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
         try:
             sess = self.sessions[session]
         except KeyError:
-            return None, constants.StatusCode.error_invalid_object
+            return None, StatusCode.error_invalid_object
 
         return sess.get_attribute(attribute)
 
@@ -304,10 +406,46 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
         try:
             sess = self.sessions[session]
         except KeyError:
-            return constants.StatusCode.error_invalid_object
+            return StatusCode.error_invalid_object
 
         return sess.set_attribute(attribute, attribute_state)
-    
+
+    def lock(self, session, lock_type, timeout, requested_key=None):
+        """Establishes an access mode to the specified resources.
+
+        Corresponds to viLock function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :param lock_type: Specifies the type of lock requested, either Constants.EXCLUSIVE_LOCK or Constants.SHARED_LOCK.
+        :param timeout: Absolute time period (in milliseconds) that a resource waits to get unlocked by the
+                        locking session before returning an error.
+        :param requested_key: This parameter is not used and should be set to VI_NULL when lockType is VI_EXCLUSIVE_LOCK.
+        :return: access_key that can then be passed to other sessions to share the lock, return value of the library call.
+        :rtype: str, :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return StatusCode.error_invalid_object
+
+        return sess.lock(lock_type, timeout, requested_key)
+
+    def unlock(self, session):
+        """Relinquishes a lock for the specified resource.
+
+        Corresponds to viUnlock function of the VISA library.
+
+        :param session: Unique logical identifier to a session.
+        :return: return value of the library call.
+        :rtype: :class:`pyvisa.constants.StatusCode`
+        """
+        try:
+            sess = self.sessions[session]
+        except KeyError:
+            return StatusCode.error_invalid_object
+
+        return sess.unlock()
+
     def disable_event(self, session, event_type, mechanism):
         # TODO: implement this for GPIB finalization
         pass
@@ -315,4 +453,3 @@ class PyVisaLibrary(highlevel.VisaLibraryBase):
     def discard_events(self, session, event_type, mechanism):
         # TODO: implement this for GPIB finalization
         pass
-
