@@ -5,7 +5,9 @@
 
     Implements Session to control USBTMC instruments
 
-    Loosely based on PyUSBTMC:python module to handle USB-TMC(Test and Measurement class)　devices.
+    Loosely based on PyUSBTMC:python module to handle USB-TMC(Test and
+    Measurement class)　devices.
+
     by Noboru Yamamot, Accl. Lab, KEK, JAPAN
 
     This file is an offspring of the Lantz Project.
@@ -14,7 +16,8 @@
     :license: MIT, see LICENSE for more details.
 """
 
-from __future__ import division, unicode_literals, print_function, absolute_import
+from __future__ import (division, unicode_literals, print_function,
+                        absolute_import)
 
 import enum
 from pyvisa.compat import struct
@@ -23,7 +26,8 @@ from collections import namedtuple
 
 import usb
 
-from .usbutil import find_devices, find_interfaces, find_endpoint, usb_find_desc
+from .usbutil import (find_devices, find_interfaces, find_endpoint,
+                      usb_find_desc)
 
 import sys
 
@@ -58,35 +62,44 @@ class Request(enum.IntEnum):
     indicator_pulse = 64
 
 
-def find_tmc_devices(vendor=None, product=None, serial_number=None, custom_match=None, **kwargs):
+def find_tmc_devices(vendor=None, product=None, serial_number=None,
+                     custom_match=None, **kwargs):
     """Find connected USBTMC devices. See usbutil.find_devices for more info.
+
     """
     def is_usbtmc(dev):
         if custom_match and not custom_match(dev):
             return False
-        return bool(find_interfaces(dev, bInterfaceClass=0xfe, bInterfaceSubClass=3))
+        return bool(find_interfaces(dev, bInterfaceClass=0xfe,
+                                    bInterfaceSubClass=3))
 
     return find_devices(vendor, product, serial_number, is_usbtmc, **kwargs)
 
 
 class BulkOutMessage(object):
-    """The Host uses the Bulk-OUT endpoint to send USBTMC command messages to the device.
+    """The Host uses the Bulk-OUT endpoint to send USBTMC command messages to
+    the device.
+
     """
 
     @staticmethod
     def build_array(btag, eom, chunk):
         size = len(chunk)
-        return struct.pack('BBBx', MsgID.dev_dep_msg_out, btag, ~btag & 0xFF) + \
-               struct.pack("<LBxxx", size, eom) + \
-               chunk + \
-               b'\0' * ((4 - size) % 4)
+        return (struct.pack('BBBx', MsgID.dev_dep_msg_out, btag,
+                            ~btag & 0xFF) +
+                struct.pack("<LBxxx", size, eom) +
+                chunk +
+                b'\0' * ((4 - size) % 4))
 
 
 class BulkInMessage(namedtuple('BulkInMessage', 'msgid btag btaginverse '
-                                                'transfer_size transfer_attributes data')):
-    """The Host uses the Bulk-IN endpoint to read USBTMC response messages from the device.
-    The Host must first send a USBTMC command message that expects a response before
-    attempting to read a USBTMC response message.
+                               'transfer_size transfer_attributes data')):
+    """The Host uses the Bulk-IN endpoint to read USBTMC response messages from
+    the device.
+
+    The Host must first send a USBTMC command message that expects a response
+    before attempting to read a USBTMC response message.
+
     """
 
     @classmethod
@@ -94,10 +107,12 @@ class BulkInMessage(namedtuple('BulkInMessage', 'msgid btag btaginverse '
         msgid, btag, btaginverse = struct.unpack_from('BBBx', data)
         assert msgid == MsgID.dev_dep_msg_in
 
-        transfer_size, transfer_attributes = struct.unpack_from('<LBxxx', data, 4)
+        transfer_size, transfer_attributes = struct.unpack_from('<LBxxx', data,
+                                                                4)
 
         data = data[12:]
-        return cls(msgid, btag, btaginverse, transfer_size, transfer_attributes, data)
+        return cls(msgid, btag, btaginverse, transfer_size,
+                   transfer_attributes, data)
 
     @staticmethod
     def build_array(btag, transfer_size, term_char=None):
@@ -115,8 +130,10 @@ class BulkInMessage(namedtuple('BulkInMessage', 'msgid btag btaginverse '
         else:
             transfer_attributes = 2
 
-        return struct.pack('BBBx', MsgID.request_dev_dep_msg_in, btag, ~btag & 0xFF) + \
-               struct.pack("<LBBxx", transfer_size, transfer_attributes, term_char)
+        return (struct.pack('BBBx', MsgID.request_dev_dep_msg_in, btag,
+                            ~btag & 0xFF) +
+                struct.pack("<LBBxx", transfer_size, transfer_attributes,
+                            term_char))
 
 
 class USBRaw(object):
@@ -126,10 +143,12 @@ class USBRaw(object):
 
     #: Configuration number to be used. If None, the default will be used.
     CONFIGURATION = None
+
     #: Interface index it be used
     INTERFACE = (0, 0)
-    #: Receive and Send endpoints to be used. If None the first IN (or OUT) BULK
-    #: endpoint will be used.
+
+    #: Receive and Send endpoints to be used. If None the first IN (or OUT)
+    #: BULK endpoint will be used.
     ENDPOINTS = (None, None)
 
     timeout = 2000
@@ -143,16 +162,17 @@ class USBRaw(object):
         self.timeout = timeout
 
         device_filters = device_filters or {}
-        devices = list(self.find_devices(vendor, product, serial_number, None, **device_filters))
+        devices = list(self.find_devices(vendor, product, serial_number, None,
+                                         **device_filters))
 
         if not devices:
             raise ValueError('No device found.')
         elif len(devices) > 1:
             desc = '\n'.join(str(dev) for dev in devices)
-            raise ValueError('{} devices found:\n{}\n'
-                             'Please narrow the search criteria'.format(len(devices), desc))
+            raise ValueError('{} devices found:\n{}\nPlease narrow the search'
+                             ' criteria'.format(len(devices), desc))
 
-        self.usb_dev, other = devices[0], devices[1:]
+        self.usb_dev = devices[0]
 
         try:
             if self.usb_dev.is_kernel_driver_active(0):
@@ -172,7 +192,8 @@ class USBRaw(object):
 
         self.usb_intf = self._find_interface(self.usb_dev, self.INTERFACE)
 
-        self.usb_recv_ep, self.usb_send_ep = self._find_endpoints(self.usb_intf, self.ENDPOINTS)
+        self.usb_recv_ep, self.usb_send_ep =\
+            self._find_endpoints(self.usb_intf, self.ENDPOINTS)
 
     def _find_interface(self, dev, setting):
         return self.usb_dev.get_active_configuration()[self.INTERFACE]
@@ -180,12 +201,14 @@ class USBRaw(object):
     def _find_endpoints(self, interface, setting):
         recv, send = setting
         if recv is None:
-            recv = find_endpoint(interface, usb.ENDPOINT_IN, usb.ENDPOINT_TYPE_BULK)
+            recv = find_endpoint(interface, usb.ENDPOINT_IN,
+                                 usb.ENDPOINT_TYPE_BULK)
         else:
             recv = usb_find_desc(interface, bEndpointAddress=recv)
 
         if send is None:
-            send = find_endpoint(interface, usb.ENDPOINT_OUT, usb.ENDPOINT_TYPE_BULK)
+            send = find_endpoint(interface, usb.ENDPOINT_OUT,
+                                 usb.ENDPOINT_TYPE_BULK)
         else:
             send = usb_find_desc(interface, bEndpointAddress=send)
 
@@ -228,9 +251,11 @@ class USBTMC(USBRaw):
 
     find_devices = staticmethod(find_tmc_devices)
 
-    def __init__(self, vendor=None, product=None, serial_number=None, **kwargs):
+    def __init__(self, vendor=None, product=None, serial_number=None,
+                 **kwargs):
         super(USBTMC, self).__init__(vendor, product, serial_number, **kwargs)
-        self.usb_intr_in = find_endpoint(self.usb_intf, usb.ENDPOINT_IN, usb.ENDPOINT_TYPE_INTERRUPT)
+        self.usb_intr_in = find_endpoint(self.usb_intf, usb.ENDPOINT_IN,
+                                         usb.ENDPOINT_TYPE_INTERRUPT)
 
         self.usb_dev.reset()
         self.usb_dev.set_configuration()
@@ -242,21 +267,23 @@ class USBTMC(USBRaw):
         self._btag = 0
 
         if not (self.usb_recv_ep and self.usb_send_ep):
-            raise ValueError("TMC device must have both Bulk-In and Bulk-out endpoints.")
+            msg = "TMC device must have both Bulk-In and Bulk-out endpoints."
+            raise ValueError(msg)
 
     def _get_capabilities(self):
-        cap = self.usb_dev.ctrl_transfer(
-                   usb.util.build_request_type(usb.util.CTRL_IN,
-                                               usb.util.CTRL_TYPE_CLASS,
-                                               usb.util.CTRL_RECIPIENT_INTERFACE),
-                   Request.get_capabilities,
-                   0x0000,
-                   self.usb_intf.index,
-                   0x0018,
-                   timeout=self.timeout)
+        self.usb_dev.ctrl_transfer(
+            usb.util.build_request_type(usb.util.CTRL_IN,
+                                        usb.util.CTRL_TYPE_CLASS,
+                                        usb.util.CTRL_RECIPIENT_INTERFACE),
+            Request.get_capabilities,
+            0x0000,
+            self.usb_intf.index,
+            0x0018,
+            timeout=self.timeout)
 
     def _find_interface(self, dev, setting):
-        interfaces = find_interfaces(dev, bInterfaceClass=0xFE, bInterfaceSubClass=3)
+        interfaces = find_interfaces(dev, bInterfaceClass=0xFE,
+                                     bInterfaceSubClass=3)
         if not interfaces:
             raise ValueError('USB TMC interface not found.')
         elif len(interfaces) > 1:
@@ -281,13 +308,13 @@ class USBTMC(USBRaw):
 
             self._btag = (self._btag % 255) + 1
 
-            data = BulkOutMessage.build_array(self._btag, end > size, data[begin:end])
+            data = BulkOutMessage.build_array(self._btag, end > size,
+                                              data[begin:end])
 
             bytes_sent += raw_write(data)
 
         return bytes_sent
 
-
     def read(self, size):
 
         recv_chunk = self.RECV_CHUNK
@@ -297,7 +324,7 @@ class USBTMC(USBRaw):
         raw_read = super(USBTMC, self).read
         raw_write = super(USBTMC, self).write
 
-        received = b''
+        received = bytearray()
 
         while not eom:
             self._btag = (self._btag % 255) + 1
@@ -310,8 +337,8 @@ class USBTMC(USBRaw):
 
             response = BulkInMessage.from_bytes(resp)
 
-            received += response.data
+            received.extend(response.data)
 
             eom = response.transfer_attributes & 1
 
-        return received
+        return bytes(received)
