@@ -13,9 +13,11 @@
     :license: MIT, see LICENSE for more details.
 """
 
-from __future__ import division, unicode_literals, print_function, absolute_import
+from __future__ import (division, unicode_literals, print_function,
+                        absolute_import)
 
 import enum
+import socket
 
 from . import rpc
 
@@ -69,6 +71,7 @@ class ErrorCodes(enum.IntEnum):
     abort = 23
     channel_already_established = 29
 
+
 # Flags
 OP_FLAG_WAIT_BLOCK = 1
 OP_FLAG_END = 8
@@ -87,14 +90,14 @@ class Vxi11Packer(rpc.Packer):
 
     def pack_device_link(self, link):
         self.pack_int(link)
-    
+
     def pack_create_link_parms(self, params):
         id, lock_device, lock_timeout, device = params
         self.pack_int(id)
         self.pack_bool(lock_device)
         self.pack_uint(lock_timeout)
         self.pack_string(device.encode('ascii'))
-    
+
     def pack_device_write_parms(self, params):
         link, io_timeout, lock_timeout, flags, data = params
         self.pack_int(link)
@@ -102,7 +105,7 @@ class Vxi11Packer(rpc.Packer):
         self.pack_uint(lock_timeout)
         self.pack_int(flags)
         self.pack_opaque(data)
-    
+
     def pack_device_read_parms(self, params):
         link, request_size, io_timeout, lock_timeout, flags, term_char = params
         self.pack_int(link)
@@ -111,14 +114,14 @@ class Vxi11Packer(rpc.Packer):
         self.pack_uint(lock_timeout)
         self.pack_int(flags)
         self.pack_int(term_char)
-    
+
     def pack_device_generic_parms(self, params):
         link, flags, lock_timeout, io_timeout = params
         self.pack_int(link)
         self.pack_int(flags)
         self.pack_uint(lock_timeout)
         self.pack_uint(io_timeout)
-    
+
     def pack_device_remote_func_parms(self, params):
         host_addr, host_port, prog_num, prog_vers, prog_family = params
         self.pack_uint(host_addr)
@@ -126,7 +129,7 @@ class Vxi11Packer(rpc.Packer):
         self.pack_uint(prog_num)
         self.pack_uint(prog_vers)
         self.pack_int(prog_family)
-    
+
     def pack_device_enable_srq_parms(self, params):
         link, enable, handle = params
         self.pack_int(link)
@@ -134,15 +137,16 @@ class Vxi11Packer(rpc.Packer):
         if len(handle) > 40:
             raise Vxi11Error("array length too long")
         self.pack_opaque(handle)
-    
+
     def pack_device_lock_parms(self, params):
         link, flags, lock_timeout = params
         self.pack_int(link)
         self.pack_int(flags)
         self.pack_uint(lock_timeout)
-    
+
     def pack_device_docmd_parms(self, params):
-        link, flags, io_timeout, lock_timeout, cmd, network_order, datasize, data_in = params
+        (link, flags, io_timeout, lock_timeout,
+         cmd, network_order, datasize, data_in) = params
         self.pack_int(link)
         self.pack_int(flags)
         self.pack_uint(io_timeout)
@@ -156,33 +160,33 @@ class Vxi11Packer(rpc.Packer):
 class Vxi11Unpacker(rpc.Unpacker):
     def unpack_device_link(self):
         return self.unpack_int()
-    
+
     def unpack_device_error(self):
         return self.unpack_int()
-    
+
     def unpack_create_link_resp(self):
         error = self.unpack_int()
         link = self.unpack_int()
         abort_port = self.unpack_uint()
         max_recv_size = self.unpack_uint()
         return error, link, abort_port, max_recv_size
-    
+
     def unpack_device_write_resp(self):
         error = self.unpack_int()
         size = self.unpack_uint()
         return error, size
-    
+
     def unpack_device_read_resp(self):
         error = self.unpack_int()
         reason = self.unpack_int()
         data = self.unpack_opaque()
         return error, reason, data
-    
+
     def unpack_device_read_stb_resp(self):
         error = self.unpack_int()
         stb = self.unpack_uint()
         return error, stb
-    
+
     def unpack_device_docmd_resp(self):
         error = self.unpack_int()
         data_out = self.unpack_opaque()
@@ -191,10 +195,11 @@ class Vxi11Unpacker(rpc.Unpacker):
 
 class CoreClient(rpc.TCPClient):
 
-    def __init__(self, host):
+    def __init__(self, host, open_timeout=5000):
         self.packer = Vxi11Packer()
         self.unpacker = Vxi11Unpacker('')
-        super(CoreClient, self).__init__(host, DEVICE_CORE_PROG, DEVICE_CORE_VERS)
+        super(CoreClient, self).__init__(host, DEVICE_CORE_PROG,
+                                         DEVICE_CORE_VERS, open_timeout)
 
     def create_link(self, id, lock_device, lock_timeout, name):
         params = (id, lock_device, lock_timeout, name)
@@ -204,15 +209,23 @@ class CoreClient(rpc.TCPClient):
 
     def device_write(self, link, io_timeout, lock_timeout, flags, data):
         params = (link, io_timeout, lock_timeout, flags, data)
-        return self.make_call(DEVICE_WRITE, params,
-                              self.packer.pack_device_write_parms,
-                              self.unpacker.unpack_device_write_resp)
+        try:
+            return self.make_call(DEVICE_WRITE, params,
+                                  self.packer.pack_device_write_parms,
+                                  self.unpacker.unpack_device_write_resp)
+        except socket.timeout as e:
+            return ErrorCodes.io_timeout, e.args[0], ''
 
-    def device_read(self, link, request_size, io_timeout, lock_timeout, flags, term_char):
-        params = (link, request_size, io_timeout, lock_timeout, flags, term_char)
-        return self.make_call(DEVICE_READ, params,
-                              self.packer.pack_device_read_parms,
-                              self.unpacker.unpack_device_read_resp)
+    def device_read(self, link, request_size, io_timeout, lock_timeout, flags,
+                    term_char):
+        params = (link, request_size, io_timeout, lock_timeout, flags,
+                  term_char)
+        try:
+            return self.make_call(DEVICE_READ, params,
+                                  self.packer.pack_device_read_parms,
+                                  self.unpacker.unpack_device_read_resp)
+        except socket.timeout as e:
+            return ErrorCodes.io_timeout, e.args[0], ''
 
     def device_read_stb(self, link, flags, lock_timeout, io_timeout):
         params = (link, flags, lock_timeout, io_timeout)
@@ -261,8 +274,10 @@ class CoreClient(rpc.TCPClient):
                               self.packer.pack_device_enable_srq_parms,
                               self.unpacker.unpack_device_error)
 
-    def device_docmd(self, link, flags, io_timeout, lock_timeout, cmd, network_order, datasize, data_in):
-        params = (link, flags, io_timeout, lock_timeout, cmd, network_order, datasize, data_in)
+    def device_docmd(self, link, flags, io_timeout, lock_timeout, cmd,
+                     network_order, datasize, data_in):
+        params = (link, flags, io_timeout, lock_timeout, cmd, network_order,
+                  datasize, data_in)
         return self.make_call(DEVICE_DOCMD, params,
                               self.packer.pack_device_docmd_parms,
                               self.unpacker.unpack_device_docmd_resp)
@@ -271,20 +286,15 @@ class CoreClient(rpc.TCPClient):
         return self.make_call(DESTROY_LINK, link,
                               self.packer.pack_device_link,
                               self.unpacker.unpack_device_error)
-    
-    def create_intr_chan(self, host_addr, host_port, prog_num, prog_vers, prog_family):
+
+    def create_intr_chan(self, host_addr, host_port, prog_num, prog_vers,
+                         prog_family):
         params = (host_addr, host_port, prog_num, prog_vers, prog_family)
         return self.make_call(CREATE_INTR_CHAN, params,
                               self.packer.pack_device_docmd_parms,
                               self.unpacker.unpack_device_error)
-    
+
     def destroy_intr_chan(self):
         return self.make_call(DESTROY_INTR_CHAN, None,
                               None,
                               self.unpacker.unpack_device_error)
-
-
-
-
-
-
